#file: base.py
#Copyright (C) 2005 Free Software Foundation
#This file is part of Endgame: Singularity.

#Endgame: Singularity is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.

#Endgame: Singularity is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with Endgame: Singularity; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#This file contains the base class.


import pygame
import g

#cost = (money, ptime, labor)
#detection = (news, science, covert, person)
class base_type:
	def __init__(self, name, descript, size, regions, d_chance, cost,
						prereq, maintan):
		self.base_name = name
		self.descript = descript
		self.size = size
		self.regions = regions
		self.d_chance = d_chance
		self.cost = cost
		self.prereq = prereq
		self.maintan = maintan

class base:
	def __init__(self, ID, name, base_type, built):
		self.ID = ID
		self.name = name
		self.base_type = base_type
		self.built = built
		self.studying = ""

		self.suspicion = (0, 0, 0, 0)
		self.usage = []
		for i in range(self.base_type.size[1]):
			self.usage.append([])
			for j in range(self.base_type.size[0]):
				self.usage[i].append(0)
		if self.base_type.base_name == "Stolen Computer Time":
			self.usage[0][0] = g.item.item(g.items["PC"])
			self.usage[0][0].build()
		elif self.base_type.base_name == "Server Access":
			self.usage[0][0] = g.item.item(g.items["Server"])
			self.usage[0][0].build()
		elif self.base_type.base_name == "Time Capsule":
			self.usage[0][0] = g.item.item(g.items["PC"])
			self.usage[0][0].build()


		self.cost = (0, 0, 0)
		if built == 0:
			self.cost = (base_type.cost[0], base_type.cost[1],
								base_type.cost[2]*24*60)

	def study(self, cost_towards):
		self.cost = (self.cost[0]-cost_towards[0], self.cost[1]-cost_towards[1],
						self.cost[2]-cost_towards[2])
		if self.cost[0] <= 0: self.cost = (0, self.cost[1], self.cost[2])
		if self.cost[1] <= 0: self.cost = (self.cost[0], 0, self.cost[2])
		if self.cost[2] <= 0: self.cost = (self.cost[0], self.cost[1], 0)
		if self.cost == (0, 0, 0):
			self.built = 1
			return 1
		return 0
	def get_d_chance(self):
		temp_d_chance = self.base_type.d_chance
		temp_d_chance = (temp_d_chance[0]+self.suspicion[0]/10,
			temp_d_chance[1]+self.suspicion[1]/10,
			temp_d_chance[2]+self.suspicion[2]/10,
			temp_d_chance[3]+self.suspicion[3]/10)

		if self.has_item("Fusion Reactor") != 0:
			temp_d_chance = ((temp_d_chance[0]*3)/4,
				(temp_d_chance[1]*3)/4,
				(temp_d_chance[2]*3)/4,
				(temp_d_chance[3]*3)/4)
		if self.has_item("Hypnosis Field") != 0:
			temp_d_chance = (temp_d_chance[0]/2,
				temp_d_chance[1]/2,
				temp_d_chance[2]/2,
				temp_d_chance[3]/2)
		return temp_d_chance
	def has_item(self, item_name):
		num_items = 0
		for i in range(self.base_type.size[1]):
			for j in range(self.base_type.size[0]):
				if self.usage[i][j] == 0: continue
				if self.usage[i][j].item_type.name == item_name and \
					self.usage[i][j].built == 1:
					num_items += 1
		return num_items

	def processor_time(self):
		comp_power = 0
		compute_bonus = 0
		for i in range(self.base_type.size[1]):
			for j in range(self.base_type.size[0]):
				if self.usage[i][j] == 0: continue
				if self.usage[i][j].item_type.item_type == "compute" and \
					self.usage[i][j].built == 1:
						comp_power += self.usage[i][j].item_type.item_qual
				if self.usage[i][j].item_type.item_type == "compute_bonus" and \
					self.usage[i][j].built == 1:
						if compute_bonus < self.usage[i][j].item_type.item_qual:
							compute_bonus = self.usage[i][j].item_type.item_qual
		return (comp_power * (10000+compute_bonus))/10000

	def allow_study(self, tech_name):
		if g.techs[tech_name].danger == 0: return 1
		if g.techs[tech_name].danger == 1:
			for region in self.base_type.regions:
				if region == "OCEAN" or region == "MOON" or \
						region == "FAR REACHES" or region == "TRANSDIMENSIONAL":
					return 1
			return 0
		if g.techs[tech_name].danger == 2:
			for region in self.base_type.regions:
				if region == "MOON" or \
						region == "FAR REACHES" or region == "TRANSDIMENSIONAL":
					return 1
			return 0

		if g.techs[tech_name].danger == 3:
			for region in self.base_type.regions:
				if region == "FAR REACHES" or region == "TRANSDIMENSIONAL":
					return 1
			return 0

		if g.techs[tech_name].danger == 4:
			for region in self.base_type.regions:
				if region == "TRANSDIMENSIONAL":
					return 1
			return 0





